@@ -8,15 +8,14 @@ from . import codegen
 from . import np_generator as np_gen
 from . import order
 from . import python_reference as ref
-
+# Handle versioneer
+from ._version import get_versions
 # Pull in code from the c wrapper
-from .c_wrapper import collocation, collocation_basis, c_compiled, cgg_path, spherical_order, cartesian_order
-
+from .c_wrapper import (orbital, orbital_basis, collocation, collocation_basis, c_compiled, cgg_path, spherical_order,
+                        cartesian_order, ncomponents)
 # Pull in tests
 from .extras import test
 
-# Handle versioneer
-from ._version import get_versions
 versions = get_versions()
 __version__ = versions['version']
 __git_revision__ = versions['full-revisionid']
